import Taro from "@tarojs/taro";
// 多端上传文件，我们得引入 PostData 和 FileData 两个类
import { axios, PostData, FileData, AxiosRequestConfig } from "taro-axios";
import { BASE_URL } from "@/config/index";
import { getToken, removeToken } from "./index";
import { authLogin } from "./auth";
import { APP_VERSION, TARO_APP_ID } from "@/config/index";
// import { TaroAdapter } from "axios-taro-adapter";
import { useAppStore } from '@/store/appStore'
import { EUploadFileScene, EUploadFileBucketName } from '@/consts/enum'

// type TResponseData = Promise<{
//   "code": number,
//   "data": string,
//   "msg": string
// }>



export const getHeaders = () => {
  const token = getToken();
  const systemInfo = useAppStore.getState().systemInfo;
  return {
    token,
    'x-app-version': APP_VERSION,
    'x-app-language': systemInfo?.language ?? 'en',
    'x-mp-app-id': TARO_APP_ID,
  };
};

// 创建请求实例
const httpConfig = {
  baseURL: BASE_URL,
};
const instance = axios.create(httpConfig);
// 添加请求拦截器
instance.interceptors.request.use(
  function (config: any) {
    const headers = getHeaders()
    config.headers = headers
    return config;
  },
  function (error: any) {
    return Promise.reject(error);
  }
);

// 添加响应拦截器
instance.interceptors.response.use(
  async function (response: any) {
    const code = response?.data?.code;
    const currentPages = Taro.getCurrentPages()
    const currentPage = currentPages[currentPages.length - 1]
    // 此处全局提示处理
    if (code !== undefined) {
      Taro.hideLoading();
      if (code === 20001) {
        // Taro.showToast({
        //   title: "登录过期",
        //   duration: 2000,
        // });
        removeToken();
        const res = await authLogin();
        if(res){
          Taro.reLaunch({
            url: `/${currentPage.$taroPath}`
          })
          console.log(currentPage)
        }
      } else if(code === 20003){
        // 资源未找到
        return Promise.resolve(response.data);
      }else if(code === 30004){
        useAppStore.getState().setVipPopupVisible(true);
        return Promise.resolve(response.data);
      }else if (code === 0) {
        // 正常业务逻辑
        return {
          ...response.data,
        };
      } else {
        const error = response.data?.msg || "网络错误";
        Taro.showModal({
          content: error,
          showCancel: false,
        });
      }
    }
    return Promise.resolve(response.data);
  },
  function (_error: any) {
    const error = _error ?? "未知错误";
    console.error(error);
    Taro.hideLoading();
    return Promise.reject(error);
  }
);

export const service = {
  get: async <T>(url: string, config?: AxiosRequestConfig) => {
    const res = instance.get(url, config) as Promise<{
      code: number;
      data: T;
      msg: string;
    }>;
    return res;
  },
  post: async <T, R = unknown>(
    url: string,
    data?: T,
    _config?: AxiosRequestConfig
  ) => {
    const res = instance.post(url, data, _config) as Promise<{
      code: number;
      data: R;
      msg: string;
    }>;
    return res;
  },
  put: async <T, R = unknown>(
    url: string,
    data?: T,
    _config?: AxiosRequestConfig
  ) => {
    const res = instance.put(url, data, _config) as Promise<{
      code: number;
      data: R;
      msg: string;
    }>;
    return res;
  },
  delete: async <T>(url: string, _config?: AxiosRequestConfig) => {
    const res = instance.delete(url, _config) as Promise<{
      code: number;
      data: T;
      msg: string;
    }>;
    return res;
  },
};

const _uploadFile = (
  tmpPath: string,
  fileName: string = "file",
  bucketName= EUploadFileBucketName.OTHER, 
  scene=EUploadFileScene.OTHER, 
  callback: (
    data: {
      code: number;
      data: string;
      msg: string;
    } | null
  ) => void
) => {
  return Taro.uploadFile({
    url: `${BASE_URL}v1/base/upload`,
    filePath: tmpPath,
    name: fileName,
    formData: {
      file: fileName,
      'scene': scene,
    },
    header: {
      ...getHeaders(),
    },
    success(res) {
      // console.log(res)
      try {
        const r = JSON.parse(res.data);
        callback && callback(r);
      } catch (e) {
        console.log(e);
        callback(null);
      }
    },
    fail(err) {
      console.log(err);
      callback(null);
    },
  });
};

export async function uploadImage(tmpPath: string, scene: EUploadFileScene = EUploadFileScene.OTHER): Promise<{
  code: number;
  data: string;
  msg: string;
} | null> {
  return new Promise((resolve) => {
    _uploadFile(tmpPath, "file", scene, (r) => {
      resolve(r);
    });
  });
}
export async function uploadFile(
  tmpPath: string,
  scene: EUploadFileScene = EUploadFileScene.OTHER
): Promise<{
  code: number;
  data: string;
  msg: string;
} | null> {
  return new Promise((resolve) => {
    _uploadFile(tmpPath, "file", scene, (data) => {
      resolve(data);
    });
  });
}


export default instance; // 导出创建后的实例
